"""
Report generation functionality
"""

import json
import yaml
from typing import Dict, Any
from datetime import datetime
from pathlib import Path

from rich.console import Console
from rich.table import Table
from rich.panel import Panel

console = Console()

class ReportGenerator:
    """Generates reports in various formats"""
    
    def __init__(self, scan_results: Dict[str, Any]):
        """Initialize with scan results"""
        self.results = scan_results
    
    def generate(self, format: str = "text") -> str:
        """Generate report in specified format"""
        if format == "text":
            return self._generate_text()
        elif format == "json":
            return self._generate_json()
        elif format == "yaml":
            return self._generate_yaml()
        else:
            raise ValueError(f"Unsupported format: {format}")
    
    def _generate_text(self) -> str:
        """Generate human-readable text report"""
        table = Table(title="System Scan Report")
        table.add_column("Category", style="cyan")
        table.add_column("Status", style="green")
        table.add_column("Details", style="white")
        
        # Add system info
        table.add_row(
            "System Info",
            "OK",
            f"Host: {self.results['system_info']['hostname']}\n"
            f"OS: {self.results['system_info']['os']} {self.results['system_info']['os_version']}\n"
            f"Arch: {self.results['system_info']['architecture']}"
        )
        
        # Add scan results
        for category, result in self.results["scan_results"].items():
            if result.get("status") == "error":
                table.add_row(category, "ERROR", result["error"])
            else:
                table.add_row(category, "OK", str(result))
        
        return str(table)
    
    def _generate_json(self) -> str:
        """Generate JSON report"""
        return json.dumps(self.results, indent=2)
    
    def _generate_yaml(self) -> str:
        """Generate YAML report"""
        return yaml.dump(self.results, default_flow_style=False) 


"""
Network scanning module
"""

import nmap
import netifaces
import socket
import subprocess
from typing import Dict, Any, List
from datetime import datetime

class NetworkModule:
    """Network scanner"""
    
    def scan(self) -> Dict[str, Any]:
        """Scan network information"""
        return {
            "interfaces": self._get_network_interfaces(),
            "open_ports": self._scan_ports(),
            "connections": self._get_connections(),
            "firewall": self._check_firewall()
        }
    
    def _get_network_interfaces(self) -> Dict[str, Any]:
        """Get network interfaces information"""
        try:
            interfaces = {}
            for iface in netifaces.interfaces():
                addrs = netifaces.ifaddresses(iface)
                interfaces[iface] = {
                    "mac": addrs.get(netifaces.AF_LINK, [{}])[0].get('addr', ''),
                    "ipv4": [addr['addr'] for addr in addrs.get(netifaces.AF_INET, [])],
                    "ipv6": [addr['addr'] for addr in addrs.get(netifaces.AF_INET6, [])]
                }
            return interfaces
        except:
            return {"error": "Could not get network interfaces"}
    
    def _scan_ports(self) -> Dict[str, Any]:
        """Scan for open ports"""
        try:
            nm = nmap.PortScanner()
            nm.scan('127.0.0.1', '20-443')
            
            open_ports = []
            for host in nm.all_hosts():
                for proto in nm[host].all_protocols():
                    ports = nm[host][proto].keys()
                    for port in ports:
                        state = nm[host][proto][port]['state']
                        if state == 'open':
                            open_ports.append({
                                "port": port,
                                "protocol": proto,
                                "service": nm[host][proto][port].get('name', 'unknown')
                            })
            
            return {"open_ports": open_ports}
        except:
            return {"error": "Could not scan ports"}
    
    def _get_connections(self) -> Dict[str, Any]:
        """Get network connections"""
        try:
            result = subprocess.run(
                ["netstat", "-tuln"],
                capture_output=True,
                text=True
            )
            
            connections = []
            for line in result.stdout.split('\n')[2:]:  # Skip header
                if line.strip():
                    parts = line.split()
                    if len(parts) >= 4:
                        proto = parts[0]
                        local = parts[3]
                        if ':' in local:
                            port = local.split(':')[-1]
                            connections.append({
                                "protocol": proto,
                                "port": port,
                                "address": local
                            })
            
            return {"connections": connections}
        except:
            return {"error": "Could not get network connections"}
    
    def _check_firewall(self) -> Dict[str, Any]:
        """Check firewall status"""
        try:
            # Check for common firewalls
            firewalls = {
                "ufw": "ufw status",
                "firewalld": "firewall-cmd --state",
                "iptables": "iptables -L"
            }
            
            status = {}
            for name, cmd in firewalls.items():
                try:
                    result = subprocess.run(cmd.split(), capture_output=True, text=True)
                    status[name] = "active" if result.returncode == 0 else "inactive"
                except:
                    status[name] = "not_installed"
            
            return status
        except:
            return {"error": "Could not check firewall status"} 


"""
Security scanning module
"""

import os
import subprocess
from typing import Dict, Any, List
from datetime import datetime

class SecurityModule:
    """Security scanner"""
    
    def scan(self) -> Dict[str, Any]:
        """Scan security information"""
        return {
            "firewall_status": self._check_firewall(),
            "open_ports": self._scan_ports(),
            "sudo_users": self._get_sudo_users(),
            "ssh_status": self._check_ssh(),
            "last_logins": self._get_last_logins(),
            "failed_logins": self._get_failed_logins()
        }
    
    def _check_firewall(self) -> Dict[str, Any]:
        """Check firewall status"""
        try:
            # Check for common firewalls
            firewalls = {
                "ufw": "ufw status",
                "firewalld": "firewall-cmd --state",
                "iptables": "iptables -L"
            }
            
            status = {}
            for name, cmd in firewalls.items():
                try:
                    result = subprocess.run(cmd.split(), capture_output=True, text=True)
                    status[name] = "active" if result.returncode == 0 else "inactive"
                except:
                    status[name] = "not_installed"
            
            return status
        except:
            return {"error": "Could not check firewall status"}
    
    def _scan_ports(self) -> Dict[str, Any]:
        """Scan for open ports"""
        try:
            # Use netstat to get listening ports
            result = subprocess.run(
                ["netstat", "-tuln"],
                capture_output=True,
                text=True
            )
            
            ports = []
            for line in result.stdout.split('\n')[2:]:  # Skip header
                if line.strip():
                    parts = line.split()
                    if len(parts) >= 4:
                        proto = parts[0]
                        address = parts[3]
                        if ':' in address:
                            port = address.split(':')[-1]
                            ports.append({
                                "protocol": proto,
                                "port": port,
                                "address": address
                            })
            
            return {"open_ports": ports}
        except:
            return {"error": "Could not scan ports"}
    
    def _get_sudo_users(self) -> List[str]:
        """Get list of users with sudo access"""
        try:
            with open('/etc/group', 'r') as f:
                for line in f:
                    if line.startswith('sudo:'):
                        return line.strip().split(':')[-1].split(',')
            return []
        except:
            return []
    
    def _check_ssh(self) -> Dict[str, Any]:
        """Check SSH service status"""
        try:
            result = subprocess.run(
                ["systemctl", "status", "ssh"],
                capture_output=True,
                text=True
            )
            return {
                "status": "active" if "active (running)" in result.stdout else "inactive",
                "details": result.stdout
            }
        except:
            return {"status": "unknown"}
    
    def _get_last_logins(self) -> List[Dict[str, str]]:
        """Get recent login attempts"""
        try:
            result = subprocess.run(
                ["last", "-n", "10"],
                capture_output=True,
                text=True
            )
            logins = []
            for line in result.stdout.split('\n'):
                if line.strip():
                    parts = line.split()
                    if len(parts) >= 4:
                        logins.append({
                            "user": parts[0],
                            "from": parts[2],
                            "when": ' '.join(parts[3:])
                        })
            return logins
        except:
            return []
    
    def _get_failed_logins(self) -> List[Dict[str, str]]:
        """Get failed login attempts"""
        try:
            result = subprocess.run(
                ["lastb", "-n", "10"],
                capture_output=True,
                text=True
            )
            failed = []
            for line in result.stdout.split('\n'):
                if line.strip():
                    parts = line.split()
                    if len(parts) >= 3:
                        failed.append({
                            "user": parts[0],
                            "from": parts[2],
                            "when": ' '.join(parts[3:])
                        })
            return failed
        except:
            return [] 


"""
Command-line interface for scanux
"""

import argparse
import json
import sys
import time
from datetime import datetime
from typing import Dict, List, Optional

from .core.scanner import SystemScanner
from .core.reporter import ReportGenerator

def parse_args() -> argparse.Namespace:
    """Parse command line arguments."""
    parser = argparse.ArgumentParser(
        description="System security and performance scanner"
    )
    parser.add_argument(
        "--json",
        action="store_true",
        help="Output results in JSON format",
    )
    parser.add_argument(
        "--yaml",
        action="store_true",
        help="Output results in YAML format",
    )
    parser.add_argument(
        "--issues-only",
        action="store_true",
        help="Show only issues, skip metrics",
    )
    parser.add_argument(
        "--modules",
        nargs="+",
        choices=["system", "security", "performance", "network"],
        default=["system", "security", "performance", "network"],
        help="Specify which modules to run",
    )
    return parser.parse_args()

def format_issues(issues: List[Dict]) -> str:
    """Format issues for human-readable output."""
    if not issues:
        return "No issues found."

    output = []
    for issue in issues:
        output.append(f"[{issue['severity']}] {issue['title']}")
        output.append(f"  Description: {issue['description']}")
        if "recommendation" in issue:
            output.append(f"  Recommendation: {issue['recommendation']}")
        output.append("")
    return "\n".join(output)

def format_metrics(metrics: Dict) -> str:
    """Format metrics for human-readable output."""
    output = []
    for category, values in metrics.items():
        output.append(f"{category}:")
        for key, value in values.items():
            output.append(f"  {key}: {value}")
        output.append("")
    return "\n".join(output)

def main() -> int:
    """Main entry point."""
    args = parse_args()
    start_time = time.time()

    try:
        # Initialize scanner
        scanner = SystemScanner(args.modules)
        results = scanner.scan()
        
        # Generate report
        reporter = ReportGenerator(results)
        format = "json" if args.json else "yaml" if args.yaml else "text"
        report = reporter.generate(format)
        
        # Print report
        print(report)
        
        # Return appropriate exit code
        return 1 if any(
            result.get("status") == "error"
            for result in results["scan_results"].values()
        ) else 0

    except Exception as e:
        print(f"Error: {str(e)}", file=sys.stderr)
        return 1

if __name__ == "__main__":
    sys.exit(main()) 


"""
Performance monitoring module
"""

import os
import psutil
from typing import Dict, Any
from datetime import datetime

class PerformanceModule:
    """Performance monitor"""
    
    def scan(self) -> Dict[str, Any]:
        """Scan performance metrics"""
        return {
            "cpu": self._get_cpu_info(),
            "memory": self._get_memory_info(),
            "disk": self._get_disk_info(),
            "network": self._get_network_info(),
            "processes": self._get_process_info()
        }
    
    def _get_cpu_info(self) -> Dict[str, Any]:
        """Get CPU information and usage"""
        try:
            return {
                "usage_percent": psutil.cpu_percent(interval=1),
                "count": psutil.cpu_count(),
                "frequency": {
                    "current": psutil.cpu_freq().current,
                    "min": psutil.cpu_freq().min,
                    "max": psutil.cpu_freq().max
                },
                "load_avg": {
                    "1min": os.getloadavg()[0],
                    "5min": os.getloadavg()[1],
                    "15min": os.getloadavg()[2]
                }
            }
        except:
            return {"error": "Could not get CPU information"}
    
    def _get_memory_info(self) -> Dict[str, Any]:
        """Get memory information and usage"""
        try:
            memory = psutil.virtual_memory()
            swap = psutil.swap_memory()
            return {
                "memory": {
                    "total": memory.total,
                    "available": memory.available,
                    "used": memory.used,
                    "free": memory.free,
                    "percent": memory.percent
                },
                "swap": {
                    "total": swap.total,
                    "used": swap.used,
                    "free": swap.free,
                    "percent": swap.percent
                }
            }
        except:
            return {"error": "Could not get memory information"}
    
    def _get_disk_info(self) -> Dict[str, Any]:
        """Get disk information and usage"""
        try:
            disk_info = {}
            for partition in psutil.disk_partitions():
                try:
                    usage = psutil.disk_usage(partition.mountpoint)
                    disk_info[partition.mountpoint] = {
                        "device": partition.device,
                        "fstype": partition.fstype,
                        "total": usage.total,
                        "used": usage.used,
                        "free": usage.free,
                        "percent": usage.percent
                    }
                except:
                    continue
            return disk_info
        except:
            return {"error": "Could not get disk information"}
    
    def _get_network_info(self) -> Dict[str, Any]:
        """Get network information and usage"""
        try:
            net_io = psutil.net_io_counters()
            return {
                "bytes_sent": net_io.bytes_sent,
                "bytes_recv": net_io.bytes_recv,
                "packets_sent": net_io.packets_sent,
                "packets_recv": net_io.packets_recv,
                "errin": net_io.errin,
                "errout": net_io.errout,
                "dropin": net_io.dropin,
                "dropout": net_io.dropout
            }
        except:
            return {"error": "Could not get network information"}
    
    def _get_process_info(self) -> Dict[str, Any]:
        """Get process information"""
        try:
            processes = []
            for proc in psutil.process_iter(['pid', 'name', 'cpu_percent', 'memory_percent']):
                try:
                    processes.append({
                        "pid": proc.info['pid'],
                        "name": proc.info['name'],
                        "cpu_percent": proc.info['cpu_percent'],
                        "memory_percent": proc.info['memory_percent']
                    })
                except:
                    continue
            return {"processes": processes}
        except:
            return {"error": "Could not get process information"} 


"""
System information module
"""

import os
import platform
import subprocess
from typing import Dict, Any
from datetime import datetime

class SystemModule:
    """System information scanner"""
    
    def scan(self) -> Dict[str, Any]:
        """Scan system information"""
        return {
            "hostname": platform.node(),
            "os": platform.system(),
            "os_version": platform.version(),
            "architecture": platform.machine(),
            "processor": platform.processor(),
            "python_version": platform.python_version(),
            "uptime": self._get_uptime(),
            "kernel": self._get_kernel_info(),
            "timezone": self._get_timezone(),
            "last_boot": datetime.fromtimestamp(self._get_boot_time()).isoformat()
        }
    
    def _get_uptime(self) -> str:
        """Get system uptime"""
        try:
            with open('/proc/uptime', 'r') as f:
                uptime_seconds = float(f.readline().split()[0])
                return str(datetime.fromtimestamp(uptime_seconds))
        except:
            return "Unknown"
    
    def _get_kernel_info(self) -> Dict[str, str]:
        """Get kernel information"""
        try:
            uname = platform.uname()
            return {
                "system": uname.system,
                "node": uname.node,
                "release": uname.release,
                "version": uname.version,
                "machine": uname.machine
            }
        except:
            return {"error": "Could not get kernel info"}
    
    def _get_timezone(self) -> str:
        """Get system timezone"""
        try:
            if os.path.exists('/etc/timezone'):
                with open('/etc/timezone', 'r') as f:
                    return f.read().strip()
            elif os.path.exists('/etc/localtime'):
                return os.readlink('/etc/localtime').split('/')[-1]
            else:
                return "Unknown"
        except:
            return "Unknown"
    
    def _get_boot_time(self) -> float:
        """Get system boot time"""
        try:
            return float(subprocess.check_output(['cat', '/proc/stat']).decode().split('\n')[0].split()[1])
        except:
            return 0.0 